#define private public
#include "../../../include/GPU/GPU.h"
#include "SimpleRead.hpp"
#include <stdio.h>

namespace FileProcessor {
    using GPU::PositionU16;
    using GPU::SizeU16;

    struct SimpleTIMSize : private SimpleTIM {
        constexpr SimpleTIMSize() {
        }

        constexpr uint16_t getTextureWidth() const {
            return SimpleTIM::getTextureX();
        }

        constexpr uint16_t getTextureHeight() const {
            return SimpleTIM::getTextureY();
        }

        constexpr uint16_t getClutWidth() const {
            return SimpleTIM::getClutX();
        }

        constexpr uint16_t getClutHeight() const {
            return SimpleTIM::getClutY();
        }
    };

    struct SimpleTIMState {
        SimpleTIM     dst_info;
        SimpleTIMSize size_info;        

        constexpr SimpleTIMState(const SimpleTIM& dst_info) : dst_info(dst_info) {
        }
    };
    static_assert(sizeof(SimpleTIMState) <= sizeof(State::Reserved));

    static void set_gpu_receive(const uint32_t* src, uint16_t x, uint16_t y, uint16_t w, uint16_t h) {
        GPU::DMA::Receive::prepare();
        GPU::DMA::Receive::set_dst(PositionU16(x, y), SizeU16(w, h)); 
        GPU::DMA::Receive::set_src(reinterpret_cast<const uintptr_t>(src));
    }

    static bool parse_header(State::Configuration& config, SimpleTIMState& state, size_t size) {
        if(size >= sizeof(SimpleTIMSize)) {
            size -= simple_read(state.size_info, config);
            
            //Check if we have a clut to care about
            if(state.size_info.getClutWidth() > 0) {
                //CLUTs are 16bit full color anyway
                printf("We found a CLUT!\n");
            }

            //We have direct data
            else {
                printf("We found direct data!\n");
            }
        }

        return false;
        static_assert(sizeof(SimpleTIMSize) == sizeof(uint32_t));
    }

    State create(const uint32_t* data_adr, const SimpleTIM& file) {        
        return State::from(SimpleTIMState(file), data_adr, parse_header);
    }
}

#undef private